using System.Collections;
using System.Collections.Generic;
using UnityEngine;

public class PlayerBehaviour : MonoBehaviour
{
    public Transform leftFoot, rightFoot;
    public Vector3 footOffset,footRotation;
    public LayerMask layerMask;
    private Animator animator;

    public LayerMask poiLayerMask;

    public Transform headTransform;

    //public Transform leftHandIKTarget, rightHandIKTarget;

    public Transform leftArm, rightArm;

    public float handOffset;
    public Vector3 handRotation;

    // Start is called before the first frame update
    void Start()
    {
        animator = GetComponent<Animator>();
    }

    // Update is called once per frame
    void Update()
    {        
        animator.SetFloat("WalkSpeed", Input.GetAxis("Vertical") * ( Input.GetKey(KeyCode.LeftShift) ? 1.0f : 0.49f ) );
        
        animator.SetFloat("Turn", Input.GetAxis("Horizontal"));

        // if (Input.GetButtonDown("Jump"))
        // {
        //     animator.SetTrigger("Kick");
        // }
    }

    void HandleIKFoot(Transform foot, AvatarIKGoal goal)
    {
        RaycastHit hitInfo;

        Debug.DrawLine(foot.transform.position+Vector3.up*0.5f,foot.transform.position,Color.red);

        if (Physics.Raycast(foot.transform.position+Vector3.up*0.5f,Vector3.down,out hitInfo,0.51f,layerMask))
        {
            animator.SetIKPosition(goal,hitInfo.point+footOffset);
            animator.SetIKRotation(goal,Quaternion.LookRotation(hitInfo.normal,transform.forward) * Quaternion.Euler(footRotation));
            animator.SetIKPositionWeight(goal,1);
            animator.SetIKRotationWeight(goal,1);
        } else
        {
            animator.SetIKPositionWeight(goal,0);
            animator.SetIKRotationWeight(goal,0);
        }
    }

    void HandleIKArm(Transform arm, AvatarIKGoal goal, string paramName)
    {
        RaycastHit hitInfo;

        Debug.DrawRay(arm.transform.position, transform.forward, Color.red);

        
        if (Physics.Raycast(arm.transform.position, transform.forward,out hitInfo,0.75f,layerMask))
        {
            animator.SetIKPosition(goal, hitInfo.point+hitInfo.normal*handOffset);
            animator.SetIKRotation(goal,Quaternion.LookRotation(Vector3.up,hitInfo.normal) * Quaternion.Euler(handRotation));
            animator.SetIKPositionWeight(goal,1);
            animator.SetIKRotationWeight(goal,1);
            animator.SetBool(paramName,true);
        } else
        {
            animator.SetIKPositionWeight(goal,0);
            animator.SetIKRotationWeight(goal,0);
            animator.SetBool(paramName,false);
        }
    }

    void OnAnimatorIK()
    {
        HandleIKFoot(leftFoot,AvatarIKGoal.LeftFoot);
        HandleIKFoot(rightFoot,AvatarIKGoal.RightFoot);

        Transform targetHeadIK = null;

        var pois = Physics.OverlapSphere(transform.position+transform.forward*2+transform.up*1.75f,2,poiLayerMask);
        if (pois.Length>0)
        {
            float minDist = float.PositiveInfinity;
            
            foreach(var poi in pois)
            {
                float dist = (headTransform.position-poi.transform.position).sqrMagnitude;

                if (dist<minDist)
                {
                    targetHeadIK = poi.transform;
                    minDist = dist;
                }
            }
        }        

        if (targetHeadIK==null)
        {
            animator.SetLookAtWeight(0);
        }
        else
        {
            animator.SetLookAtWeight(1);
            animator.SetLookAtPosition(targetHeadIK.position);
        }

        /*animator.SetIKPosition(AvatarIKGoal.LeftHand, leftHandIKTarget.position);
        animator.SetIKPosition(AvatarIKGoal.RightHand, rightHandIKTarget.position);
        
        animator.SetIKRotation(AvatarIKGoal.LeftHand, leftHandIKTarget.rotation);
        animator.SetIKRotation(AvatarIKGoal.RightHand, rightHandIKTarget.rotation);

        animator.SetIKPositionWeight(AvatarIKGoal.LeftHand, leftHandIKTarget.gameObject.activeSelf ? 1 : 0);
        animator.SetIKPositionWeight(AvatarIKGoal.RightHand, rightHandIKTarget.gameObject.activeSelf ? 1 : 0);
        
        animator.SetIKRotationWeight(AvatarIKGoal.LeftHand, leftHandIKTarget.gameObject.activeSelf ? 1 : 0);
        animator.SetIKRotationWeight(AvatarIKGoal.RightHand, rightHandIKTarget.gameObject.activeSelf ? 1 : 0);*/

		HandleIKArm(leftArm,AvatarIKGoal.LeftHand,"LeftHandOnWall");
		HandleIKArm(rightArm,AvatarIKGoal.RightHand,"RightHandOnWall");
    }

    void OnDrawGizmos()
    {
        
        Gizmos.color = Color.red;
        Gizmos.DrawWireSphere(transform.position+transform.forward*2+transform.up*1.75f,2);    
    }
}
